﻿/**
 * @class Node
 *
 * @description Node (abstract data type node)
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 *
 */

"use strict";

function Node(oData){
	var self = this;
	
	/** @public */
	this.tParent = undefined;								//Parent node of this instance
	
	/** Array of node instsance childrens
	 * @public
	 * @type Array
	 * @default new Array();
	 */
	this.children = new Array();
	
	/** @public */
	this.data = oData;
	
	/** Add children node
	 * @param {Node} oChild Node to be added to current node as children
	 */
	this.addChild = function(oChild){
        //--console.log("Add child");
		//Check node instance
		if(!oChild instanceof Node){ console.error("oChild is not a Node instance"); return undefined; }
		
		//console.warn(oChild.tParent);
		if(oChild.tParent != null){
			oChild.tParent.removeChild(oChild);			//Remove new child parent if exist
		}
		oChild.tParent = self;							//Set current node as parent
		self.children.push(oChild);						//Push new child node in childrens list
	};
	
	/** Remove child node from parent
	 * @param {Node} oChild Children node to be removed
	 */
	this.removeChild = function(oChild){
        //--console.log("Remove child");
		var index = self.children.indexOf(oChild);		//Get index of target child
		if(index == -1){ return 1; }					//Return if child doesn't exist in childrens array list
		var obj = self.children.splice(index,1);		//Remove children from childrens array list
		obj[0].tParent = undefined;					    //Unset parent from deleted child
		
		return 0;
	};
	
	/** Shift child node from parent
	 * @param {Integer} intIndex integer for new index of childs array
	 */
	this.shiftNode = function(strIndex){
        //--console.log("Shift node");
		var intIndex = parseInt(strIndex,10);
		
		//Check
		if(isNaN(intIndex) || intIndex == undefined || intIndex == null || intIndex < 0){
			console.warn("Invalid intIndex parameter: "+ strIndex);
			return 1;
		}
		
		var oParent = self.getParent();
		
		if(oParent == undefined || oParent == null){
			console.error("Shift node does not belong to any parent node");
			return 2;
		}
		
		var aChilds = oParent.children;
		var intOldIndex = aChilds.indexOf(self);
		
		aChilds.splice(intIndex,0,aChilds.splice(intOldIndex,1)[0]);
	};
	
	/** Set node data
	 * @param {Node} oData object or whatever kind of data
	 */
	this.setData = function(oData){
        //--console.log("Set data");
		self.data = oData;
	};
	
	/** Get node data
	 * @returns {Object} Data object
	 */
	this.getData = function(){
        //--console.log("Get Data");
		return self.data;
	};
	
	/** Get parent
	 * @returns {Node} Parent node
	 */
	this.getParent = function(){
        //--console.log("Get Parent");
		return self.tParent;
	};
	
	/** Is root
	 */
	this.isRoot = function(){
		return (self.tParent == undefined);
	};
	
	/** JSON serialization
     *
     */
    this.encodeJSON = function(){
        //--console.log("Encode json");
        //Set instance variables
        return self.data.encodeJSON();
    };

    /** Return a string contains node dump
     * If == undefined than it will be "data" (eg. Node["data"]).
     * If aWalk is a String Node[aWalk] parameter will be read.
     * @returns String of Node hierarchy
     */
    this.getDeep = function(){
        //--console.log("get deep");
        var deep = 0;
        var walk = self;

        while(!( walk.isRoot())){
            walk = walk.getParent();
            deep++;
        }

        return deep;
    };
	
	/** Return a string contains node dump
	 * @param aWalk array contains string for walk to desidered parameter inside node (eg. Node[aWalk[0]][aWalk[1]][...]).
	 * If == undefined than it will be "data" (eg. Node["data"]).
	 * If aWalk is a String Node[aWalk] parameter will be read.
	 * @returns String of Node hierarchy
	 */
	this.dump = function(aWalk){
        //--console.log("dump");
		var concData = self;
		var strReturn = "";
		
		//Check input param type
		if(aWalk == undefined){ concData = concData["data"]; }
		else if(typeof aWalk === "string"){
			concData = concData[aWalk];
		}
		else if(aWalk instanceof Array){
			for(var i=0; i<aWalk.length; ++i){
				concData = concData[aWalk[i]];
				if(concData == undefined){ concData = "(no param)"; break; }
			}
		}
		
		concData = self.toString();
		
		//strReturn += concData +"\n";
		//strReturn += concData;
		
		//if(self.children.length > 0){ strReturn += ": \n\t"; }
		
		//if(self.children.length < 1){ strReturn = strReturn; }
		
		//Start recursion
		//var temp = new Array();
		
		if(self.children.length > 0){
			//strReturn += concData +":\n";
			strReturn += concData +":";
			
			for(var i=0; i<self.children.length; ++i){
				//strReturn += "\n\t "+ self.children[i].dump(aWalk);
				strReturn += "\n\t"+ self.children[i].dump(aWalk);
				
				//if(i<self.children.length - 1){ strReturn +="\n"; }
			}
		}
		//else{ strReturn += "\t"+ concData; }
		else{ strReturn += concData +"\n"; }

		return strReturn;
	};
	
	/** Overwrite toString() method
     * @returns {string} String that describe Node object
     */
    this.toString = function(){
        //--console.log("node tostring");
        return "[Node] "+ self.data.toString();
    };
};